var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

/**
* Parallax.js
* @author Matthew Wagerfield - @wagerfield, René Roth - mail@reneroth.org, Grégoire Sage
* @description Creates a parallax effect between an array of layers,
*              driving the motion from the gyroscope output of a smartdevice.
*/

import { Accelerometer } from "accelerometer";
import { display } from 'display';

var helpers = {
  clamp: function clamp(value, min, max) {
    return min < max ? value < min ? min : value > max ? max : value : value < max ? max : value > min ? min : value;
  }
};

var DEFAULTS = {
  calibrationThreshold: 100,
  calibrationDelay: 500,
  calibrateX: false,
  calibrateY: true,
  invertX: true,
  invertY: true,
  limitX: false,
  limitY: false,
  scalarX: 10.0,
  scalarY: 10.0,
  frictionX: 0.1,
  frictionY: 0.1
};

export var Parallax = function () {
  function Parallax(element, options) {
    _classCallCheck(this, Parallax);

    this.element = element;

    for (var key in DEFAULTS) {
      this[key] = DEFAULTS[key];
    }
    for (var key in options) {
      this[key] = options[key];
    }

    this.calibrationTimer = null;
    this.calibrationFlag = true;
    this.enabled = false;
    this.wasenabled = false;
    this.raf = null;

    this.elementWidth = 0;
    this.elementHeight = 0;

    this.calibrationX = 0;
    this.calibrationY = 0;

    this.inputX = 0;
    this.inputY = 0;

    this.motionX = 0;
    this.motionY = 0;

    this.velocityX = 0;
    this.velocityY = 0;

    this.onAnimationFrame = this.onAnimationFrame.bind(this);
    this.onAccelerometerSensor = this.onAccelerometerSensor.bind(this);
    this.onCalibrationTimer = this.onCalibrationTimer.bind(this);
    this.onDisplayChanged = this.onDisplayChanged.bind(this);

    this.initialise();

    display.addEventListener('change', this.onDisplayChanged);
  }

  _createClass(Parallax, [{
    key: 'initialise',
    value: function initialise() {
      // Setup
      this.updateLayers();
      this.enable();
      this.queueCalibration(this.calibrationDelay);
    }
  }, {
    key: 'updateLayers',
    value: function updateLayers() {
      // if(!this.element.firstChild) {
      //   console.warn('ParallaxJS: Your scene does not have any layers.')
      // }

      this.layers = [];
      this.depths = [];

      var layer = this.element.firstChild;
      while (layer) {
        this.layers.push(layer);
        this.depths.push(layer.layer / 100 || 0);
        layer = layer.nextSibling;
      }
    }
  }, {
    key: 'updateBounds',
    value: function updateBounds() {
      this.elementWidth = this.element.width;
      this.elementHeight = this.element.height;
    }
  }, {
    key: 'queueCalibration',
    value: function queueCalibration(delay) {
      clearTimeout(this.calibrationTimer);
      this.calibrationTimer = setTimeout(this.onCalibrationTimer, delay);
    }
  }, {
    key: 'enable',
    value: function enable() {
      if (this.enabled) {
        return;
      }
      this.enabled = true;

      this.sensor = new Accelerometer({ frequency: 10 });
      this.sensor.onreading = this.onAccelerometerSensor;
      this.detectionTimer = setTimeout(this.onAccelerometerTimer, this.supportDelay);

      this.raf = requestAnimationFrame(this.onAnimationFrame);
      this.sensor.start();
    }
  }, {
    key: 'disable',
    value: function disable() {
      if (!this.enabled) {
        return;
      }
      this.enabled = false;

      this.sensor.stop();
      delete this.sensor;
      cancelAnimationFrame(this.raf);
    }
  }, {
    key: 'calibrate',
    value: function calibrate(x, y) {
      this.calibrateX = x === undefined ? this.calibrateX : x;
      this.calibrateY = y === undefined ? this.calibrateY : y;
    }
  }, {
    key: 'invert',
    value: function invert(x, y) {
      this.invertX = x === undefined ? this.invertX : x;
      this.invertY = y === undefined ? this.invertY : y;
    }
  }, {
    key: 'friction',
    value: function friction(x, y) {
      this.frictionX = x === undefined ? this.frictionX : x;
      this.frictionY = y === undefined ? this.frictionY : y;
    }
  }, {
    key: 'scalar',
    value: function scalar(x, y) {
      this.scalarX = x === undefined ? this.scalarX : x;
      this.scalarY = y === undefined ? this.scalarY : y;
    }
  }, {
    key: 'limit',
    value: function limit(x, y) {
      this.limitX = x === undefined ? this.limitX : x;
      this.limitY = y === undefined ? this.limitY : y;
    }
  }, {
    key: 'setPosition',
    value: function setPosition(element, x, y) {
      element.groupTransform.translate.x = x;
      element.groupTransform.translate.y = y;
    }
  }, {
    key: 'onCalibrationTimer',
    value: function onCalibrationTimer() {
      this.calibrationFlag = true;
    }
  }, {
    key: 'onAnimationFrame',
    value: function onAnimationFrame() {
      var _this = this;

      this.updateBounds();
      var calibratedInputX = this.inputX - this.calibrationX,
          calibratedInputY = this.inputY - this.calibrationY;
      if (Math.abs(calibratedInputX) > this.calibrationThreshold || Math.abs(calibratedInputY) > this.calibrationThreshold) {
        this.queueCalibration(0);
      }
      {
        this.motionX = this.calibrateX ? calibratedInputX : this.inputX;
        this.motionY = this.calibrateY ? calibratedInputY : this.inputY;
      }
      this.motionX *= this.elementWidth * (this.scalarX / 100);
      this.motionY *= this.elementHeight * (this.scalarY / 100);
      if (!isNaN(parseFloat(this.limitX))) {
        this.motionX = helpers.clamp(this.motionX, -this.limitX, this.limitX);
      }
      if (!isNaN(parseFloat(this.limitY))) {
        this.motionY = helpers.clamp(this.motionY, -this.limitY, this.limitY);
      }
      this.velocityX += (this.motionX - this.velocityX) * this.frictionX;
      this.velocityY += (this.motionY - this.velocityY) * this.frictionY;
      this.layers.forEach(function (layer, index) {
        var depth = _this.depths[index],
            xOffset = _this.velocityX * (depth * (_this.invertX ? -1 : 1)),
            yOffset = _this.velocityY * (depth * (_this.invertY ? -1 : 1));
        _this.setPosition(layer, xOffset, yOffset);
      });
      this.raf = requestAnimationFrame(this.onAnimationFrame);
    }
  }, {
    key: 'onAccelerometerSensor',
    value: function onAccelerometerSensor() {
      var x = this.sensor.x;
      var y = -this.sensor.y;

      if (this.calibrationFlag) {
        this.calibrationFlag = false;
        this.calibrationX = x;
        this.calibrationY = y;
      }

      this.inputX = x;
      this.inputY = y;
    }
  }, {
    key: 'onDisplayChanged',
    value: function onDisplayChanged() {
      if (display.on) {
        if (this.wasenabled) {
          this.enable();
        }
      } else {
        this.wasenabled = this.enabled;
        if (this.enabled) {
          this.disable();
        }
      }
    }
  }, {
    key: 'destroy',
    value: function destroy() {
      this.disable();

      clearTimeout(this.calibrationTimer);
      clearTimeout(this.detectionTimer);

      delete this.element;
      delete this.layers;
    }
  }]);

  return Parallax;
}();